--[[
_____/\\\\\\\\\____________________________________/\\\\\\\\\___________________
 ___/\\\\\\\\\\\\\________________________________/\\\///////\\\_________________
  __/\\\/////////\\\______________________________\/\\\_____\/\\\_________________
   _\/\\\_______\/\\\__/\\\____/\\\_____/\\\\\\\\__\/\\\\\\\\\\\/________/\\\\\____
    _\/\\\\\\\\\\\\\\\_\///\\\/\\\/____/\\\/////\\\_\/\\\//////\\\______/\\\///\\\__
     _\/\\\/////////\\\___\///\\\/_____/\\\\\\\\\\\__\/\\\____\//\\\____/\\\__\//\\\_
      _\/\\\_______\/\\\____/\\\/\\\___\//\\///////___\/\\\_____\//\\\__\//\\\__/\\\__
       _\/\\\_______\/\\\__/\\\/\///\\\__\//\\\\\\\\\\_\/\\\______\//\\\__\///\\\\\/___
        _\///________\///__\///____\///____\//////////__\///________\///_____\/////_____ ]]
--
local Tunnel = module('vrp', 'lib/Tunnel')
local Proxy = module('vrp', 'lib/Proxy')
vRP = Proxy.getInterface("vRP")
vRPclient = Tunnel.getInterface("vRP", "astrix_faction")


local resName = GetCurrentResourceName()

-- callbacks

local userDuttys = {}
local factionMenu = {}
AddEventHandler('onResourceStart', function(rs)
    if rs == resName then
        Citizen.CreateThread(function()
            local dbDATA = exports.oxmysql:executeSync('SELECT id,duty_time,faction FROM vrp_users');
            for k, v in pairs(dbDATA) do
                userDuttys[v.id] = {
                    time = v.duty_time,
                    active = false,
                    faction = v.faction
                }
            end
            local fData = exports.oxmysql:executeSync('SELECT * FROM astrix_factionmenu');
            if fData and #fData > 0 then
                for k, v in pairs(fData) do
                    factionMenu[v.faction] = v;
                    factionMenu[v.faction].armamentStock = json.decode(factionMenu[v.faction].armamentStock)
                end
            end

            for k, v in pairs(Armament) do
                if not factionMenu[k] then
                    exports.oxmysql:execute(
                        'INSERT INTO astrix_factionmenu (faction,moneymade,hasArmament,armamentStock) VALUES (@faction,0,true,@stock)',
                        {
                            faction = k,
                            stock = json.encode({})
                        })
                    factionMenu[v.faction] = {
                        moneymade = 0,
                        armamentStock = {}

                    }
                    print('New faction created in astrix_factionmenu SQL table (' .. k .. ')')
                end
            end
        end)
    end
end)

AddEventHandler('onResourceStop', function(rs)
    if rs == resName then
        for k, v in pairs(vRP.getUsers({})) do
            local user_id = vRP.getUserId({ v });
            if user_id then
                if userDuttys[user_id] then
                    exports.oxmysql:execute('UPDATE vrp_users SET duty_time = @duty WHERE id = @user_id',
                        { user_id = user_id, duty = userDuttys[user_id].time })
                end
            end
        end
    end
end)

function registerCallback(cbName, cb)
    RegisterServerEvent(resName .. ":s_callback:" .. cbName, function(...)
        local player = source
        if cb and player then
            local result = table.pack(cb(player, ...))
            TriggerClientEvent(resName .. ":c_callback:" .. cbName, player, table.unpack(result))
        end
    end)
end

-- functions

local function giveFactionMoney(faction, money)
    if faction and money and factionMenu[faction] then
        factionMenu[faction].moneymade = factionMenu[faction].moneymade + money;
        exports.oxmysql:execute('UPDATE astrix_factionmenu SET moneymade = moneymade + @value WHERE faction = @faction',
            { faction = faction, value = money })
    end
end

exports('giveFactionMoney', giveFactionMoney);


--RegisterCommand('givefm', function()
--    exports['astrix_factionmenu']:giveFactionMoney('Politie', 1200)
--end)
--

local function formatTime(minutes)
    if not minutes then return 0 end;
    if minutes == 0 then
        return "0 min"
    end

    local hours = math.floor(minutes / 60)
    local remainingMinutes = minutes % 60

    if hours > 0 then
        return string.format("%dh %dmin", hours, remainingMinutes)
    else
        return string.format("%d min", remainingMinutes)
    end
end

local function getRoleplayName(user_id, source)
    local response = promise.new();
    vRP.getUserIdentity({ user_id, function(identity)
        if identity then
            response:resolve(identity.firstname);
        else
            response:resolve(GetPlayeName(source));
        end
    end })
    Citizen.Await(response);
    return response.value;
end

RegisterCommand('fdt', function(source)
    print('duty time', json.encode(userDuttys[vRP.getUserId({ source })]))
end)

local function getFactionMembers(faction)
    local table = {};
    local members = vRP.getUsersByFaction({ faction })
    if members then
        for k, v in pairs(members) do
            if v.id then
                local time = 0
                if userDuttys[v.id] and userDuttys[v.id].time ~= nil then
                    time = userDuttys[v.id].time
                end
                table[#table + 1] = {
                    id = v.id,
                    name = v.username,
                    duty_time = formatTime(time) or '0 min',
                    online = ((vRP.getUserSource({ v.id }) and true) or false),
                    factionRank = v.factionRank
                }
            end
        end
    end
    return table;
end

local function getOnlineRanksNumber(faction, rank)
    local nr = 0;
    if faction and rank then
        local users = vRP.getOnlineUsersByFaction({ faction })
        if users then
            for k, v in pairs(users) do
                if vRP.getFactionRank({ tonumber(v), faction }) == rank then
                    nr = nr + 1;
                end
            end
        end
    end
    return nr;
end


local function getFactionRanks(faction)
    local table = {}
    local ranks = vRP.getFactionRanks({ faction })
    if ranks then
        for k, v in pairs(ranks) do
            table[#table + 1] = {
                label = v.rank,
                salary = v.salary,
                online = getOnlineRanksNumber(faction, v.rank),
            }
        end
    end
    return table;
end

RegisterServerEvent('astrix_factionmenu:tryOpenMenu', function(user_id, source)
    if user_id and source then
        local userFaction = vRP.getUserFaction({ user_id });
        if userFaction and userFaction ~= 'user' then
            if not userDuttys[user_id] then
                userDuttys[user_id] = {
                    active = false,
                    time = 0,
                    faction = userFaction
                }
            end
            local playerData = {
                user_id = user_id,
                name = getRoleplayName(user_id, source),
                isLeader = vRP.isFactionLeader({ user_id, userFaction }),
               -- isCoLider = vRP.isFactionCoLider({ user_id, userFaction }),
                activeDuty = userDuttys[user_id].active or false
            }
            local factionData = {
                faction        = userFaction,
                factionType    = vRP.getFactionType({ userFaction }),
                totalMembers   = #(vRP.getUsersByFaction({ userFaction })),
                max_slots      = vRP.getFactionSlots({ userFaction }),
                online_players = (#(vRP.getOnlineUsersByFaction({ userFaction }))),
                members        = getFactionMembers(userFaction),
                ranks          = getFactionRanks(userFaction)
            }
            local armamentStock, moneymade = nil, 0
            if factionMenu[userFaction] then
                armamentStock = factionMenu[userFaction].armamentStock or {};
                moneymade = factionMenu[userFaction].moneymade or 0
            end
            TriggerClientEvent('astrix_factionmenu:openMenu', source, factionData, playerData,
                Armament[userFaction] or nil, armamentStock, moneymade)
        end
    end
end)

RegisterCommand('fmenu', function(source)
    local user_id = vRP.getUserId({ source });
    if user_id then
        TriggerEvent('astrix_factionmenu:tryOpenMenu', user_id, source);
    end
end)


RegisterServerEvent('astrix_factionmenu:promoteMember', function(memberId, faction)
    local src = source;
    local user_id = vRP.getUserId({ src });
    if user_id then
        if vRP.isUserInFaction({ user_id, faction }) and (vRP.isFactionLeader({ user_id, faction }) or vRP.isFactionCoLeader({ user_id, faction })) then
            local memberSrc = vRP.getUserSource({ memberId });
            if memberSrc then
                local name = GetPlayerName(memberSrc);
                if (vRP.isUserInFaction({ memberId, faction })) then
                    local oldRank = vRP.getFactionRank({ memberId })
                    if vRP.factionRankUp({ memberId }) then
                        local newRank = vRP.getFactionRank({ memberId })
                        vRPclient.notify(src,
                            { " L-ai promovat pe " .. name .. "  de la " .. oldRank .. " la " .. newRank .. "!" })
                        vRPclient.notify(memberSrc,
                            { " Ai fost promovat de la " ..
                            oldRank .. " la " .. newRank .. " in factiunea " .. faction .. "!" })
                    else
                        vRPclient.notify(src, { " " .. name .. " are deja cel mai mare rank!" })
                    end
                else
                    vRPclient.notify(src, { " Jucatorul " .. name .. " nu este membru in factiunea " ..
                    faction .. "!" })
                end
            else
                vRPclient.notify(src, { 'Jucatorul trebuie sa fie online', 'error' })
            end
        else
            vRPclient.notify(src, { 'Nu ai access la aceasta functie', 'error' })
        end
    end
end)


RegisterServerEvent('astrix_factionmenu:unpromoteMember', function(memberId, faction)
    local src = source;
    local user_id = vRP.getUserId({ src });
    if user_id then
        if vRP.isUserInFaction({ user_id, faction }) and (vRP.isFactionLeader({ user_id, faction }) or vRP.isFactionCoLeader({ user_id, faction })) then
            local memberSrc = vRP.getUserSource({ memberId });
            if memberSrc then
                local name = GetPlayerName(memberSrc);
                if (vRP.isUserInFaction({ memberId, faction })) then
                    local oldRank = vRP.getFactionRank({ memberId })
                    local oldRank = vRP.getFactionRank({ memberId })
                    if (vRP.factionRankDown({ memberId })) then
                        local newRank = vRP.getFactionRank({ memberId })
                        vRPclient.notify(src, { " L-ai retrogradat pe " ..
                        name .. "  de la " .. oldRank .. " la " .. newRank .. "!" })
                        vRPclient.notify(memberSrc,
                            { " Ai fost retrogradat la " .. oldRank .. " la " .. newRank ..
                            " in factiunea " .. faction .. "!" })
                    else
                        vRPclient.notify(src, { " " .. name .. " are deja cel mai mic rank!" })
                    end
                else
                    vRPclient.notify(src, { " Jucatorul " .. name .. " nu este membru in factiunea " ..
                    faction .. "!" })
                end
            else
                vRPclient.notify(src, { 'Jucatorul trebuie sa fie online', 'error' })
            end
        else
            vRPclient.notify(src, { 'Nu ai access la aceasta functie', 'error' })
        end
    end
end)

RegisterServerEvent('astrix_factionmenu:removeMember', function(id, faction)
    local src = source;
    local user_id = vRP.getUserId({ src });
    if user_id then
        if vRP.isUserInFaction({ user_id, faction }) and (vRP.isFactionLeader({ user_id, faction }) or vRP.isFactionCoLeader({ user_id, faction })) then
            vRP.removeUserFaction({ id, faction })
            vRPclient.notify(src, { 'Jucatorul a fost scos cu sucess din factiune', 'info' })
        else
            vRPclient.notify(src, { 'Nu ai access la aceasta functie', 'error' })
        end
    end
end)

RegisterServerEvent('astrix_factionmenu:promoteLider', function(id, faction)
    local src = source;
    local user_id = vRP.getUserId({ src });
    if user_id then
        if vRP.isUserInFaction({ user_id, faction }) and (vRP.isFactionLeader({ user_id, faction })) then
            local target = vRP.getUserSource({ id })
            if target then
                if (vRP.isUserInFaction({ id, faction })) then
                    vRPclient.notify(src, { "~w~L-ai promovat pe ~g~" ..
                    GetPlayerName(target) .. " ~w~la ~g~Lider!" })
                    vRPclient.notify(src,
                        { "~w~Functia Lider ti s-a fost scoasa ~g~pentru ca ai dat Liderul Altcuiva!" })
                    vRPclient.notify(target, { "~w~Ai fost promovat la ~g~Lider ~w~in factiunea ~g~" .. faction ..
                    "!" })
                    vRP.setFactionLeader({ id })
                    vRP.setFactionNonLeader({ user_id })
                else
                    vRPclient.notify(src,
                        { "~w~Jucatorul ~g~" ..
                        GetPlayerName(target) .. " ~w~nu este membru in factiunea ~g~" .. faction .. "!" })
                end
            else
                vRPclient.notify(src, { 'Jucatorul trebuie sa fie online', 'error' })
            end
        else
            vRPclient.notify(src, { 'Nu ai access la aceasta functie', 'error' })
        end
    end
end)

RegisterServerEvent('astrix_factionmenu:addMember', function(faction)
    local src = source;
    local user_id = vRP.getUserId({ src });
    if user_id then
        if vRP.isUserInFaction({ user_id, faction }) and (vRP.isFactionLeader({ user_id, faction }) or vRP.isFactionCoLeader({ user_id, faction })) then
            local theFaction = faction
            local members = vRP.getUsersByFaction({ theFaction })
            local fSlots = vRP.getFactionSlots({ faction })
            vRP.prompt({ src, "Jucator ID: ", "", function(player, id)
                local id = parseInt(id)
                if (tonumber(id)) and (id > 0) and (id ~= "") and (id ~= nil) then
                    if (tonumber(#members) < tonumber(fSlots)) then
                        local target = vRP.getUserSource({ id })
                        if (target) then
                            if (vRP.hasUserFaction({ id })) then
                                vRPclient.notify(player,
                                    { "~r~" .. vRP.getPlayerName({ target }) .. " este deja intr-o factiune!" })
                                return
                            else
                                vRPclient.notify(player,
                                    { "~w~L-ai adaugat pe ~g~" ..
                                    vRP.getPlayerName({ target }) .. " ~w~in ~g~" .. theFaction .. "!" })
                                vRPclient.notify(target, { "~w~Ai fost adaugat in ~g~" .. theFaction .. "!" })
                                vRP.addUserFaction({ id, theFaction })
                                vRP.addUserGroup({ id, "onduty" })
                            end
                        else
                            vRP.addUserFactionOffline({ id, theFaction })
                            vRPclient.notify(player,
                                { "~w~L-ai adaugat offline pe ~g~" ..
                                id .. " ~w~in ~g~" .. theFaction .. "!" })
                        end
                    else
                        vRPclient.notify(player,
                            { "~r~Maximul de jucatori in factiune a fost atins! Sloturi: " .. fSlots })
                    end
                else
                    vRPclient.notify(player, { "~r~Acest ID este invalid" })
                end
            end })
        else
            vRPclient.notify(src, { 'Nu ai access la aceasta functie', 'error' })
        end
    end
end)


RegisterServerEvent('astrix_factionmenu:resetDuty', function(faction)
    local src = source;
    local user_id = vRP.getUserId({ src });
    if user_id then
        if vRP.isUserInFaction({ user_id, faction }) and (vRP.isFactionLeader({ user_id, faction }) or vRP.isFactionCoLeader({ user_id, faction })) then
            for k, v in pairs(vRP.getUsers({})) do
                local id = vRP.getUserId({ v });
                if id then
                    if userDuttys[id] then
                        userDuttys[id].time = 0
                        userDuttys[id].active = false
                    end
                end
            end
            exports.oxmysql:execute('UPDATE vrp_users SET duty_time = 0 WHERE faction = @faction', { faction = faction });
            vRPclient.notify(src, { 'Ai resetat toate Duty Timer-urile', 'success' })
        else
            vRPclient.notify(src, { 'Nu ai access la aceasta functie', 'error' })
        end
    end
end)

local function isUserOnDuty(user_id, faction)
    print('is user on duty')
    print(user_id, faction)
    if user_id and faction then
        if userDuttys[user_id] then
            return userDuttys[user_id].active or false;
        end
    end
    return false;
end

exports('isUserOnDuty', isUserOnDuty);

RegisterServerEvent('astrix_factionmenu:toogleDuty', function()
    local src = source;
    local user_id = vRP.getUserId({ src });
    if user_id then
        if userDuttys[user_id] then
            if userDuttys[user_id].active == false then
                userDuttys[user_id].active = true;
                vRPclient.notify(src, { 'Te-ai pus on-duty', 'info' })
                Citizen.CreateThread(function()
                    while userDuttys[user_id].active and vRP.getUserSource({ user_id }) do
                        Citizen.Wait(60 * 1000)
                        if userDuttys[user_id].active and vRP.getUserSource({ user_id }) then
                            userDuttys[user_id].time = userDuttys[user_id].time + 1;
                        end
                    end
                end)
                local userF = vRP.getUserFaction({ user_id });
                if vRP.getFactionType({ userF }) == 'Lege' then
                    local rank = vRP.getFactionRank({ user_id })
                    TriggerClientEvent('astrix_faction:dressRankUniform', src, userF, rank);
                end
            else
                vRPclient.notify(src, { 'Te-ai pus off-duty', 'info' })
                userDuttys[user_id].active = false;
                TriggerClientEvent('astrix_faction:undressRankUniform', src);
            end
        end
    end
end)

RegisterServerEvent('astrix_factionmenu:leaveFaction', function(faction)
    local src = source;
    local user_id = vRP.getUserId({ src });
    if user_id then
        if vRP.isUserInFaction({ user_id, faction }) then
            vRP.removeUserFaction({ user_id, faction })
            vRP.removeFactionFromUserFully({ user_id, faction })
            if userDuttys[user_id] then
                userDuttys[user_id].active = false;
            end
            vRPclient.notify(src, { 'Ai parasi cu sucess factiunea ' .. faction, 'success' });
        end
    end
end)

AddEventHandler("vRP:playerLeave", function(user_id)
    if user_id then
        if userDuttys[user_id] then
            if userDuttys[user_id].active then
                userDuttys[user_id].active = false;
                exports.oxmysql:execute('UPDATE vrp_users SET duty_time = @duty WHERE id = @user_id',
                    { user_id = user_id, duty = userDuttys[user_id].time })
            end
        end
    end
end)



-- update

local function findItemDataByID(item, faction)
    if not Armament[faction] then
        return;
    end
    for i = 1, #Armament[faction] do
        if Armament[faction][i].item == item then
            return Armament[faction][i]
        end
    end
end

RegisterServerEvent('astrix_factionmenu:buyWeaponStock', function(item, faction)
    local src = source;
    local user_id = vRP.getUserId({ src });
    if user_id then
        if vRP.isUserInFaction({ user_id, faction }) and vRP.isFactionLeader({ user_id, faction }) then
            local itemCfg = findItemDataByID(item, faction);
            if not itemCfg then
                return print('error 1 to buy armament')
            end
            vRP.prompt({ src, 'Cat stock doresti sa cumperi?', '', function(player, amount)
                if amount then
                    amount = tonumber(amount)
                    if type(amount) == 'number' then
                        amount = math.floor(amount);
                        local totalPrice = amount * itemCfg.priceForStock;
                        if factionMenu[faction].moneymade >= totalPrice then
                            factionMenu[faction].moneymade = factionMenu[faction].moneymade - totalPrice;
                            if not factionMenu[faction].armamentStock[item] then
                                factionMenu[faction].armamentStock[item] = 0;
                            end
                            factionMenu[faction].armamentStock[item] = factionMenu[faction].armamentStock[item] + amount;
                            exports.oxmysql:execute(
                                'UPDATE astrix_factionmenu SET moneymade = moneymade - @value, armamentStock = @stock WHERE faction = @faction',
                                {
                                    faction = faction,
                                    value = totalPrice,
                                    stock = json.encode(factionMenu[faction]
                                        .armamentStock)
                                })
                        else
                            vRPclient.notify(src, { 'Factiunea ta nu are destui bani salvati' })
                        end
                    end
                end
            end })
        else
            vRPclient.notify(src, { 'Nu ai access man3' })
        end
    end
end)

RegisterServerEvent('astrix_factionmenu:equipWeapon', function(item, faction)
    local src = source;
    local user_id = vRP.getUserId({ src });
    if user_id then
        if vRP.isUserInFaction({ user_id, faction }) and vRP.isFactionLeader({ user_id, faction }) then
            local itemCfg = findItemDataByID(item, faction);
            if not itemCfg then
                return print('error 1 to buy armament')
            end
            if factionMenu[faction] then
                if factionMenu[faction].armamentStock and factionMenu[faction].armamentStock[item] then
                    if factionMenu[faction].armamentStock[item] > 0 then
                        vRP.prompt({ src, 'Cat stock doresti sa cumperi?', '', function(player, amount)
                            if amount then
                                amount = tonumber(amount)
                                if type(amount) == 'number' then
                                    amount = math.floor(amount);
                                    if factionMenu[faction].armamentStock[item] >= amount then
                                        if exports.axr_inventory:getUserMaxWeight(user_id) >= exports.axr_inventory:getUserWeight(user_id) + (exports.axr_inventory:getItemWeight(item) * amount) then
                                            exports.axr_inventory:givePlayerItem(user_id, item, amount, true);
                                            factionMenu[faction].armamentStock[item] = factionMenu[faction]
                                                .armamentStock
                                                [item] - amount;
                                            exports.oxmysql:execute(
                                                'UPDATE astrix_factionmenu SET armamentStock = @stock WHERE faction = @faction',
                                                {
                                                    faction = faction,
                                                    stock = json.encode(factionMenu[faction]
                                                        .armamentStock)
                                                })
                                        else
                                            vRPclient.notify(src, { 'Nu ai destul spatiu in inventar' })
                                        end
                                    else
                                        vRPclient.notify(src, { 'Nu este destul stock la arma respectiva' })
                                    end
                                end
                            end
                        end })
                    else
                        vRPclient.notify(src, { 'Nu este destul stock la arma respectiva' })
                    end
                else
                    vRPclient.notify(src, { 'Nu este destul stock la arma respectiva' })
                end
            end
        end
    end
end)
